@@ -3,92 +3,43 @@
 pytest_sugar
 ~~~~~~~~~~~~
 
-py.test plugin to show failures instantly.
+py.test is a plugin for py.test that changes the default look
+and feel of py.test (e.g. progressbar, show tests that fail instantly).
 
 :copyright: see LICENSE for details
 :license: BSD, see LICENSE for more details.
 """
-import pytest
-import pytest, py
+from __future__ import unicode_literals
+import py
 import sys
-import random
 import os
-import json
 import time
-import re
+
+import pytest
 from _pytest.terminal import TerminalReporter
 
 
-class TerminalColors:
-    HEADER = '\033[95m'
-    OKBLUE = '\033[94m'
-    OKGREEN = '\033[92m'
-    GRAY = '\033[1;30m'
-    WARNING = '\033[93m'
-    FAIL = '\033[91m'
-    ENDC = '\033[0m'
+__version__ = '0.3.0'
 
-    def disable(self):
-        self.HEADER = ''
-        self.OKBLUE = ''
-        self.OKGREEN = ''
-        self.WARNING = ''
-        self.FAIL = ''
-        self.ENDC = ''
-        self.GRAY = ''
-
-
-bcolors = TerminalColors()
-
-class EtaLogger:
-    def __init__(self):
-        self.settings_path = os.path.expanduser("~/.pytest-sugar.json")
-        try:
-            self.collected = json.loads(
-                open(self.settings_path).read()
-            )
-        except IOError:
-            self.collected = {}
-        #print "-----------------"
-        #print self.collected
-
-    def collect(self, time, filename=None, hash=None):
-        self.collected[filename] = time
-        self.save()
-
-    def estimate(self, paths):
-        estimated_time = 0
-        for path in paths:
-            if path in self.collected:
-                estimated_time += self.collected[path]
-        return estimated_time
-
-    def save_session(
-        self,
-        tests_duration,
-        session_duration
-    ):
-        tests_sum = 0
-        for path, time in tests_duration.items():
-            self.collect(
-                filename=path,
-                time=time
-            )
-            tests_sum += time
-
-        extra_time = session_duration - tests_sum
-        self.collect(
-            filename=os.getcwd(),
-            time=extra_time
-        )
-        self.save()
-
-
-    def save(self):
-        #print "WRITING TO ",self.settings_path
-        fp = open(self.settings_path, 'w')
-        fp.write(json.dumps(self.collected))
-        fp.close()
+LEN_RIGHT_MARGIN = 5
+LEN_PROGRESS_BAR = 10
+LEN_SPACE_BETWEEN_PERCENT_AND_PROGRESS_BAR = 1
+LEN_PERCENT = 3
+LEN_SPACE_BETWEEN_TEST_STATUS_AND_PERCENT = 4
+TERMINAL_COLORS = {
+    'header': '\033[95m',
+    'okblue': '\033[94m',
+    'okgreen': '\033[92m',
+    'gray': '\033[1;30m',
+    'gray_bg': '\033[100m',
+    'warning': '\033[93m',
+    'fail': '\033[91m',
+    'endc': '\033[0m'
+}
+PROGRESS_BAR_BLOCKS = [
+    '█', '▉', '▉', '▊', '▊', '▋', '▋', '▌',
+    '▌', '▍', '▍', '▎', '▎', '▏', '▏'
+]
 
 
 def flatten(l):
@@ -101,12 +52,18 @@ def flatten(l):
 
 
 def pytest_collection_modifyitems(session, config, items):
-    config.pluginmanager.getplugin('terminalreporter').tests_count += len(items)
+    if config.option.sugar:
+        terminal_reporter = config.pluginmanager.getplugin('terminalreporter')
+        terminal_reporter.tests_count += len(items)
+
 
 def pytest_deselected(items):
     """ Update tests_count to not include deselected tests """
     if len(items) > 0:
-        items[0].config.pluginmanager.getplugin('terminalreporter').tests_count -= len(items)
+        pluginmanager = items[0].config.pluginmanager
+        terminal_reporter = pluginmanager.getplugin('terminalreporter')
+        terminal_reporter.tests_count -= len(items)
+
 
 def pytest_addoption(parser):
     group = parser.getgroup("terminal reporting", "reporting", after="general")
@@ -119,63 +76,49 @@ def pytest_addoption(parser):
 
 
 def real_string_length(string):
-    for color in [
-        bcolors.HEADER,
-        bcolors.OKBLUE,
-        bcolors.OKGREEN,
-        bcolors.GRAY,
-        bcolors.WARNING,
-        bcolors.FAIL,
-        bcolors.ENDC
-    ]:
-        string = string.replace(color, '')
+    for color_name, color_string in TERMINAL_COLORS.items():
+        string = string.replace(color_string, '')
     return len(string)
 
+
 @pytest.mark.trylast
 def pytest_configure(config):
+    global pytest_report_teststatus
+
     if config.option.sugar:
         # Get the standard terminal reporter plugin and replace it with our
         standard_reporter = config.pluginmanager.getplugin('terminalreporter')
-        sugar_reporter = InstafailingTerminalReporter(standard_reporter)
+        sugar_reporter = SugarTerminalReporter(standard_reporter)
         config.pluginmanager.unregister(standard_reporter)
         config.pluginmanager.register(sugar_reporter, 'terminalreporter')
+        pytest_report_teststatus = _pytest_report_teststatus
 
 
-def pytest_report_teststatus(report):
+def _pytest_report_teststatus(report):
     if report.passed:
-        letter = bcolors.OKGREEN+u'✓'+bcolors.ENDC
+        letter = TERMINAL_COLORS['okgreen']+'✓'+TERMINAL_COLORS['endc']
     elif report.skipped:
-        letter = bcolors.OKBLUE+u's'+bcolors.ENDC
+        letter = TERMINAL_COLORS['okblue']+'s'+TERMINAL_COLORS['endc']
     elif report.failed:
-        letter = bcolors.FAIL+u'⨯'+bcolors.ENDC
+        letter = TERMINAL_COLORS['fail']+'⨯'+TERMINAL_COLORS['endc']
         if report.when != "call":
-            letter = bcolors.FAIL+u'ₓ'+bcolors.ENDC
+            letter = TERMINAL_COLORS['fail']+'ₓ'+TERMINAL_COLORS['endc']
     return report.outcome, letter, report.outcome.upper()
 
-class InstafailingTerminalReporter(TerminalReporter):
+
+class SugarTerminalReporter(TerminalReporter):
     def __init__(self, reporter):
-        #pytest_collectreport = self.pytest_collectreport
         TerminalReporter.__init__(self, reporter.config)
         self.writer = self._tw
-        self.eta_logger = EtaLogger()
         self.paths_left = []
         self.tests_count = 0
         self.tests_taken = 0
-        self.current_line = u''
+        self.current_line = ''
         self.currentfspath2 = ''
         self.time_taken = {}
         self.reports = []
         self.unreported_errors = []
 
-    def get_estimate(self):
-        session_duration = py.std.time.time() - self._sessionstarttime
-
-        estimate = 0
-        estimate += self.eta_logger.estimate(self.paths_left)
-        estimate += self.eta_logger.estimate([os.getcwd()])
-        estimate -= session_duration
-        return estimate
-
     def report_collect(self, final=False):
         pass
 
@@ -192,7 +135,12 @@ class InstafailingTerminalReporter(TerminalReporter):
     def pytest_sessionstart(self, session):
         self._sessionstarttime = py.std.time.time()
         verinfo = ".".join(map(str, sys.version_info[:3]))
-        self.write_line("Test session starts (%s, %s)" % (sys.platform, verinfo), bold=True)
+        self.write_line(
+            "Test session starts "
+            "(platform: %s, Python %s, pytest %s, pytest-sugar %s)" % (
+                sys.platform, verinfo, pytest.__version__, __version__,
+            ), bold=True
+        )
         lines = self.config.hook.pytest_report_header(config=self.config)
         lines.reverse()
         for line in flatten(lines):
@@ -202,40 +150,24 @@ class InstafailingTerminalReporter(TerminalReporter):
         return
 
     def insert_progress(self):
-        def get_estimate():
-            #return str(round(self.get_estimate(),2))
-            seconds = int(round(self.get_estimate()))
-            minutes = seconds/60
-            seconds = seconds%60
-            return "%im %is" % (minutes, seconds)
-
         def get_progress_bar():
-            blocks = [
-                u'█',
-                u'▉',
-                u'▉',
-                u'▊',
-                u'▊',
-                u'▋',
-                u'▋',
-                u'▌',
-                u'▌',
-                u'▍',
-                u'▍',
-                u'▎',
-                u'▎',
-                u'▏',
-                u'▏'
-            ]
-            length = 10
+            length = LEN_PROGRESS_BAR
             p = float(self.tests_taken) / self.tests_count
-            floored = int(round(p * length))
-            progressbar = u''
+            floored = int(p * length)
+            rem = int(round((p * length - floored) * 13))
+            progressbar = ''
             progressbar += "%i%% " % round(p*100)
-            progressbar += bcolors.OKGREEN
-            progressbar += blocks[0] * floored
-            progressbar += bcolors.ENDC
-            progressbar += bcolors.GRAY + blocks[0] * (length - floored) + bcolors.ENDC
+            progressbar += TERMINAL_COLORS['okgreen']
+            progressbar += TERMINAL_COLORS['gray_bg']
+            progressbar += PROGRESS_BAR_BLOCKS[1] * floored
+            if p == 1.0:
+                progressbar += PROGRESS_BAR_BLOCKS[1]
+            else:
+                progressbar += PROGRESS_BAR_BLOCKS[14 - rem]
+            progressbar += TERMINAL_COLORS['endc']
+            progressbar += TERMINAL_COLORS['gray'] + TERMINAL_COLORS['gray_bg']
+            progressbar += PROGRESS_BAR_BLOCKS[1] * (length - floored)
+            progressbar += TERMINAL_COLORS['endc']
             return progressbar
 
         append_string = get_progress_bar()
@@ -243,13 +175,40 @@ class InstafailingTerminalReporter(TerminalReporter):
 
     def append_string(self, append_string=''):
         console_width = self._tw.fullwidth
-        full_line = self.current_line + " " * (console_width  - real_string_length(self.current_line))
+        num_spaces = console_width - real_string_length(self.current_line)
+        full_line = self.current_line + " " * num_spaces
         full_line = full_line[0:-(len(append_string)-15)] + append_string
-        return full_line  
+        return full_line
 
     def overwrite(self, line):
-        sys.stdout.write("\r" + self.append_string(line).encode('utf-8'))
-        sys.stdout.flush()
+        self.writer.write("\r" + self.append_string(line))
+
+    def begin_new_line(self, report, print_filename):
+        basename = os.path.basename(report.fspath)
+        if print_filename:
+            self.current_line = (
+                "   " +
+                TERMINAL_COLORS['gray'] +
+                report.fspath[0:-len(basename)] +
+                TERMINAL_COLORS['endc'] +
+                report.fspath[-len(basename):] +
+                " "
+            )
+        else:
+            self.current_line = " " * (4 + len(report.fspath))
+        print("")
+
+    def reached_last_column_for_test_status(self):
+        max_column_for_test_status = (
+            self._tw.fullwidth
+            - LEN_RIGHT_MARGIN
+            - LEN_PROGRESS_BAR
+            - LEN_SPACE_BETWEEN_PERCENT_AND_PROGRESS_BAR
+            - LEN_PERCENT
+            - LEN_SPACE_BETWEEN_TEST_STATUS_AND_PERCENT
+        )
+        len_line = real_string_length(self.current_line)
+        return len_line == max_column_for_test_status
 
     def pytest_runtest_logreport(self, report):
         self.reports.append(report)
@@ -267,27 +226,18 @@ class InstafailingTerminalReporter(TerminalReporter):
             if not path in self.time_taken:
                 self.time_taken[path] = 0
             self.time_taken[path] += time_taken
+            if self.reached_last_column_for_test_status():
+                self.begin_new_line(report, print_filename=False)
         if report.when == 'call':
             if report.fspath != self.currentfspath2:
                 self.currentfspath2 = report.fspath
-                basename = os.path.basename(report.fspath)
-                self.current_line = (
-                    u"   " +
-                    bcolors.GRAY +
-                    report.fspath[0:-len(basename)] +
-                    bcolors.ENDC +
-                    report.fspath[-len(basename):] +
-                    u" "
-                )
-                print("")
+                self.begin_new_line(report, print_filename=True)
 
             rep = report
             res = pytest_report_teststatus(report=report)
             cat, letter, word = res
             self.current_line = self.current_line + letter
 
-
-
             self.stats.setdefault(cat, []).append(rep)
             if not letter and not word:
                 return
@@ -296,11 +246,11 @@ class InstafailingTerminalReporter(TerminalReporter):
                     word, markup = word
                 else:
                     if rep.passed:
-                        markup = {'green':True}
+                        markup = {'green': True}
                     elif rep.failed:
-                        markup = {'red':True}
+                        markup = {'red': True}
                     elif rep.skipped:
-                        markup = {'yellow':True}
+                        markup = {'yellow': True}
                 line = self._locationline(str(rep.fspath), *rep.location)
                 if not hasattr(rep, 'node'):
                     self.write_ensure_prefix(line, word, **markup)
@@ -321,20 +271,18 @@ class InstafailingTerminalReporter(TerminalReporter):
     def summary_stats(self):
         session_duration = py.std.time.time() - self._sessionstarttime
 
-        self.eta_logger.save_session(self.time_taken, session_duration)
-
-        print("\nResults (%.2fs):" % round(session_duration,2))
+        print("\nResults (%.2fs):" % round(session_duration, 2))
         if self.count('passed') > 0:
             self.write_line(
-                "   %d passed" % self.count('passed') + 
-                bcolors.ENDC
+                "   %d passed" % self.count('passed') +
+                TERMINAL_COLORS['endc']
             )
 
         if self.count('failed') > 0:
             self.write_line(
-                bcolors.FAIL+
-                "   %d failed" % self.count('failed') + 
-                bcolors.ENDC
+                TERMINAL_COLORS['fail'] +
+                "   %d failed" % self.count('failed') +
+                TERMINAL_COLORS['endc']
             )
             for report in self.reports:
                 if report.outcome == 'failed':
@@ -344,19 +292,18 @@ class InstafailingTerminalReporter(TerminalReporter):
 
         if self.count('skipped') > 0:
             self.write_line(
-                bcolors.GRAY+
-                "   %d skipped" % self.count('skipped') + 
-                bcolors.ENDC
+                TERMINAL_COLORS['fail'] +
+                "   %d skipped" % self.count('skipped') +
+                TERMINAL_COLORS['endc']
             )
 
         if self.count('deselected') > 0:
             self.write_line(
-                bcolors.GRAY+
-                "   %d deselected" % self.count('deselected') + 
-                bcolors.ENDC
+                TERMINAL_COLORS['fail'] +
+                "   %d deselected" % self.count('deselected') +
+                TERMINAL_COLORS['endc']
             )
 
-
     def summary_failures(self):
         # Prevent failure summary from being shown since we already
         # show the failure instantly after failure has occured.
@@ -381,5 +328,5 @@ class InstafailingTerminalReporter(TerminalReporter):
                 elif report.when == "teardown":
                     msg = "ERROR at teardown of " + msg
                 self.write_line('')
-                self.write_sep(u"―", msg)
+                self.write_sep("―", msg)
                 self._outrep_summary(report)
